#! /usr/bin/env python2.7

from scapy.all import *
from netfilterqueue import NetfilterQueue
import sys			# for stdin.read and sys.argv
import socket		# for inet_ntoa and inet_aton
import string		# for string.split

nfq_num = 1
hops={}

def nfq_cb(packet):
  pkt = IP(packet.get_payload()) # convert the NF packet to a scapy packet
  packet.drop()	# drop the original NF packet as we have a copy
#  print "src = _%s_ dst = _%s_ ttl = _%s_" % (pkt.src, pkt.dst, pkt.ttl)
  if pkt.ttl in hops:	# find TTL in the hops we're spoofing
    hop=hops[pkt.ttl];
    if hop == "*":		# don't send any replies
#      print "Got * for TTL %s. Dropping packet.." % pkt.ttl
      pass
    else:
# extract IP header and first 8 bytes of probing packet. Should include options header, hence .ihl
      inhdr=str(pkt)[:(8 + pkt[IP].ihl * 4)]
      if pkt.ttl < len(hops):	# not our last hop so build ICMP time exceeded response
#        print "Sending time exceeded packet from %s for TTL %s.." % (hop,pkt.ttl)
        p=IP(src=hop,dst=pkt.src)/ICMP(type=11,code=0)/inhdr
      else:	# last hop so build ICMP port unreachable response
#        print "Sending port unreachable from %s for TTL %s.." % (hop,pkt.ttl)
        p=IP(src=hop,dst=pkt.src)/ICMP(type=3,code=3)/inhdr
# The newly formed response packet has an invalid checksum
# so delete checksums and rebuild the packet with checksums
      del(p[ICMP].chksum)
      del(p[IP].chksum)
      p=p.__class__(str(p))
      send(p)	# send spoofed packet to networking stack
  else:
#    print "TTL not found."
    pass

ttl=1
if len(sys.argv) == 2:
    try:
	  nfq_num = int(sys.argv[1])
    except ValueError:
	  print "usage: %s [Netfilter Queue number (integer)]" % sys.argv[0]
	  exit(1)
for destip in string.split(sys.stdin.read()):
  try:
# check input is valid IPv4 address
    dip=socket.inet_aton(destip)
    hops[ttl] = socket.inet_ntoa(dip)
    print "%s: %s" % (ttl,hops[ttl])
    ttl+=1
  except socket.error:
    if destip == "*":
      hops[ttl] = "*";
      print "%s: %s" % (ttl,hops[ttl])
      ttl+=1
    else:
      print "%s was not a valid IPv4 address. Exiting.." % destip
      exit(1)
nfq = NetfilterQueue()
nfq.bind(nfq_num, nfq_cb) 
try:
  print "Waiting for packets on netfilter queue number %d.." % nfq_num
  nfq.run()
except KeyboardInterrupt:
  pass

